------
--
-- SetBaseScorecards
--
-- Sets base scorecards for Metrics scorecards and applies inheritance options to each base scorecard
--	with current inheritance options defined in the model.
--
-- Input:
-- #BaseUrl		Base URL for the QPR web service endpoint
-- #ModelId		Model identifier
-- #SessionData	Session identifier
-- #SCData		A temporary table consisting of ElementId NVARCHAR(80) and BaseSCSymbol NVARCHAR(80) columns.
--				The rows in the table will be deleted during the execution of this script!
--
---------------------------------------------------------------------------

-- This script can be only called from another script.
(SELECT 'MessageText', 'Base URL must be set before calling SetBaseScorecards.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#BaseUrl') IS NOT NULL,'0','1'))
--#Exit 

(SELECT 'MessageText', 'Temporary table #ModelId must be set before calling SetBaseScorecards.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#ModelId') IS NOT NULL,'0','1'))
--#Exit 

-- SessionData must exists.
(SELECT 'MessageText', 'Can not find session data (#SessionData temporary table not set)!') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#SessionData') IS NOT NULL,'0','1'))
--#Exit 

(SELECT 'MessageText', 'Temporary table #SCData must be set before calling SetBaseScorecards!') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#SCData') IS NOT NULL,'0','1'))
--#Exit 

-- Exit immediately if no data is found from #SCData
(SELECT 'Exit', IIF(CONVERT(INT,COUNT(*)) > 0,'0','1') FROM #SCData)
--#Exit 

DECLARE @baseSCSymbol AS NVARCHAR(MAX);
DECLARE @elementIds AS NVARCHAR(MAX);
SELECT TOP(1) @baseSCSymbol = CONVERT(NVARCHAR(MAX), [BaseSCSymbol]) FROM #SCData;

IF OBJECT_ID('tempdb..#BaseSCSymbol') IS NOT NULL
DROP TABLE #BaseSCSymbol;
SELECT @baseSCSymbol AS VAL INTO #BaseSCSymbol;

-- Construct a comma-delimited string of element identifiers.
SELECT @elementIds = COALESCE(@elementIds + ',', '') + [ElementId]
FROM #SCData
WHERE [BaseSCSymbol] = @baseSCSymbol;

IF OBJECT_ID('tempdb..#ElementIds') IS NOT NULL
DROP TABLE #ElementIds;
SELECT @elementIds AS VAL INTO #ElementIds;

-- Exit and do recursion if no elementids were actually found.
(SELECT 'Exit', IIF(LEN(COALESCE(@elementIds, '')) > 0,'0','1'))
(SELECT 'RunScriptId', @_CurrentScriptId)
--#Exit 

DECLARE @SessionString AS NVARCHAR(MAX);
DECLARE @ModelId AS NVARCHAR(MAX);
DECLARE @Body AS NVARCHAR(MAX);

DECLARE @baseSCSymbol NVARCHAR(MAX)= (SELECT VAL FROM #BaseSCSymbol);
DECLARE @elementIds NVARCHAR(MAX)= (SELECT VAL FROM #ElementIds);

-- Remove data to be handled from the temporary table.
DELETE FROM #SCData
WHERE [BaseSCSymbol] = @baseSCSymbol;

------------------------------------------------------------------------------------------
-- Fetch the SC identifier (SC.xxx.yyy) according to symbol given in @baseSCSymbol
------------------------------------------------------------------------------------------
PRINT 'baseSCSymbol = ' + @baseSCSymbol
SELECT @SessionString = [SES_ID] FROM #SessionData;
SELECT @ModelId = [VAL] FROM #ModelId;
SELECT @Body = '{"sessionId":"' + @SessionString + '",'
	+ '"query":"[' + @ModelId + '].Scorecard(criteria=\"Find(\\\"' + @baseSCSymbol + '\\\",Symbol)\")",'
	+ ' "criteria":"", "sortBy":"", "attributes":"Id, Name, Symbol" ,"options":""}';

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', (SELECT VAL FROM #BaseUrl) + '/QueryObjectsAsXML') UNION ALL
(SELECT 'Body', @Body) UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'CatchOperationExceptions', '1') UNION ALL
(SELECT 'Accept', '*/*')
--#CallWebService

DECLARE @BaseUrl NVARCHAR(MAX);
SELECT @BaseURL = CONVERT(NVARCHAR(MAX), [VAL]) FROM #BaseUrl;

IF OBJECT_ID('tempdb..#ResponseText') IS NOT NULL
DROP TABLE #ResponseText;
SELECT @_ResponseText AS VAL INTO #ResponseText;
	
-- If CallWebService was successful continue, else exit
(SELECT 'MessageText', 'QueryObjectsAsXML failed!') UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,@_ResponseSuccess) = 1,'0','1'))
--#Exit 

DECLARE @pos INT;
DECLARE @ResponseText NVARCHAR(MAX) = (SELECT VAL FROM #ResponseText);
DECLARE @xmlString NVARCHAR(MAX);
DECLARE @x xml;

PRINT @ResponseText

-- Extract ResultSet XML
SET @pos = PATINDEX('%"d":"%',@ResponseText);
IF @pos > 0
 BEGIN
	SET @xmlString = REPLACE(SUBSTRING(@ResponseText, @pos + 5, (LEN(@ResponseText)-8)), '\"', '"');
	SET @xmlString = REPLACE(@xmlString, '\u000d', '');
	SET @xmlString = REPLACE(@xmlString, '\u000a', '');
	SET @xmlString = REPLACE(@xmlString, '\/', '/');
  
	--  PRINT 'XML ResultSet Len: ' + CONVERT(VARCHAR, LEN(@ResponseText))
	PRINT 'XML ResultSet: ' + @xmlString;
 END;

SET @x = @xmlString

DECLARE @results AS NVARCHAR(MAX);
DECLARE @baseSCId AS NVARCHAR(MAX);

SELECT @results = @x.value('(/ResultSetHierarchy/@totalResults)[1]', 'int');
SELECT @baseSCId = @x.value('(/ResultSetHierarchy/object/@Id)[1]', 'varchar(100)');

DECLARE @elementIds AS NVARCHAR(MAX);
SELECT @elementIds = CONVERT(NVARCHAR(MAX), [VAL]) FROM #ElementIds;
 
PRINT 'Results: ' + CONVERT(VARCHAR,@results);
PRINT 'BaseSCId: ' + CONVERT(VARCHAR,@baseSCId);	
PRINT 'ElementIds: ' + @elementIds;	

IF OBJECT_ID('tempdb..#BaseSCId') IS NOT NULL
DROP TABLE #BaseSCId;
SELECT @baseSCId AS VAL INTO #BaseSCId;

-- Set the basescorecardid attribute
DECLARE @SessionString AS NVARCHAR(MAX);
DECLARE @Body AS NVARCHAR(MAX);

SELECT @SessionString = [SES_ID] FROM #SessionData;
SELECT @Body = '{"sessionId":"' + @SessionString + '",'
	+ '"objectIds":"' + @elementIds + '",'
	+ '"attribute":"basescorecardid",'
	+ '"value":"' + @baseSCId + '",'
	+ '"options":""}';

PRINT 'SetAttribute: Body: [' + @Body + ']'

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', (SELECT VAL FROM #BaseUrl) + '/SetAttribute') UNION ALL
(SELECT 'Body', @Body) UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'Accept', '*/*') UNION ALL
(SELECT 'CatchOperationExceptions', '1')
--#CallWebService

PRINT 'SetAttribute ResponseSuccess ' + CONVERT(VARCHAR, @_ResponseSuccess)
PRINT 'SetAttribute ResponseText ' + CONVERT(NVARCHAR(MAX), @_ResponseText)

-- If CallWebService was successful continue, else exit
(SELECT 'MessageText', 'SetAttribute failed while trying to set base scorecard identifiers!') UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,@_ResponseSuccess) = 1,'0','1'))
--#Exit 

-- Apply inheritance options through PerformObjectOperation
DECLARE @SessionString AS NVARCHAR(MAX);
DECLARE @Body AS NVARCHAR(MAX);
DECLARE @baseSCId AS NVARCHAR(MAX);

SELECT @SessionString = [SES_ID] FROM #SessionData;
SELECT @baseSCId = [VAL] FROM #BaseSCId;
SELECT @Body = '{"sessionId":"' + @SessionString + '",'
	+ '"objectId":"' + @baseSCId + '",'
	+ '"operation":"apply",'
	+ '"options":""}';

PRINT 'PerformObjectOperation: Body: [' + @Body + ']'

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', (SELECT VAL FROM #BaseUrl) + '/PerformObjectOperation') UNION ALL
(SELECT 'Body', @Body) UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'Accept', '*/*') UNION ALL
(SELECT 'CatchOperationExceptions', '1')
--#CallWebService

PRINT 'PerformObjectOperation ResponseSuccess ' + CONVERT(VARCHAR, @_ResponseSuccess)
PRINT 'PerformObjectOperation ResponseText ' + CONVERT(NVARCHAR(MAX), @_ResponseText)

-- Exit the script if no more data is available in #SCData
(SELECT 'MessageText', 'Base scorecards set.') UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,COUNT(*)) > 0,'0','1') FROM #SCData)
--#Exit 

-- Recursively call this script again.
(SELECT 'ScriptId', @_CurrentScriptId)
--#Run